//
// Irrlicht Wrapper for Imperative Languages
// Frank Dodd (2006)
//
// This wrapper DLL encapsulates a sub-set of the features of the powerful
// Irrlicht 3D Graphics Engine exposing the Object Oriented architecture and
// providing a functional 3D SDK for languages that are not object oriented.
//
// This source was created with the help of the great examples in the Irrlicht
// SDK and the excellent Irrlicht documentation. This software was developed
// using the GCC compiler and Code::Blocks IDE
//

/* ////////////////////////////////////////////////////////////////////////////
includes
*/
#include "irrlichtwrapper.h"

//bool StackedEventReceiver::OnEvent( SEvent  event){}; // added be me
//#include "CCloudSceneNode.h"
//#include "CTerrainSphereNode.h"

/* ////////////////////////////////////////////////////////////////////////////
Global variables
*/
IrrlichtDevice *device;
IVideoDriver* driver;
ISceneManager* smgr;
IGUIEnvironment* guienv;

// The Instance of the Event Receiver Class
StackedEventReceiver receiver;


/* ////////////////////////////////////////////////////////////////////////////
Global Function Declarations

all of the below functions are delared as C functions and are exposed without
any mangled names so that they can be easily imported into unstructured
languages like FreeBasic
*/
extern "C"
{

/* ////////////////////////////////////////////////////////////////////////////
all of the above functions are delared as C functions and are exposed without
any mangled names
*/
}

extern "C" __declspec(dllexport) void Irrclose(void)
{
	device->closeDevice();
return ;
}


extern "C" __declspec(dllexport) void Irrsetnoderotation123
              (ISceneNode * testNode,f32 const o1,f32 const o2,f32 const o3) 
{ 
   quaternion rot1;
   rot1.fromAngleAxis(o1*DEGTORAD,vector3df(0,1,0));
   quaternion rot2;
   rot2.fromAngleAxis(o2*DEGTORAD,vector3df(0,0,1));
   quaternion rot3;
   rot3.fromAngleAxis(o3*DEGTORAD,vector3df(1,0,0));
   quaternion rot123=rot3*(rot2*rot1);
   
   vector3df eulers; 
   rot123.toEuler(eulers); //... in radians 
   eulers *= RADTODEG;   //... and now in degrees 
   testNode->setRotation(eulers); 
      
return;
} 


extern "C" __declspec(dllexport) void irrdraw2dline(s32 const x0,s32 const y0,s32 const x1,s32 const y1,
                                 u32 const A,u32 const R,u32 const G,u32 const B )
{
       position2d<s32> xy0=position2d<s32>(x0,y0);
       position2d<s32> xy1=position2d<s32>(x1,y1);
       driver->draw2DLine(xy0,xy1,SColor(A,R,G,B) );   
return ;
} 


extern "C" __declspec(dllexport) void  irrdraw2dimagescale(
                                video::ITexture * texture,
                                int TX,int TY,int BX,int BY,
                                int TX2,int TY2,int BX2,int BY2,
                                bool usealpha )
{
    driver->draw2DImage(
            texture,
            rect<s32>(TX2,TY2,BX2,BY2),
            rect<s32>(TX,TY,BX,BY),
            NULL,
            NULL,usealpha);
return ;
}

extern "C" __declspec(dllexport) void  irrwriteimagetofile( IImage * image, const char * file )
{
       bool retcode;
       retcode=device->getVideoDriver()->writeImageToFile( image, file );

return ;
}       

extern "C" __declspec(dllexport) void  irrgettexturesize( video::ITexture * texture, int * x, int * y)
{       
		const dimension2d<s32> * dimxy = &texture->getSize();
        *x=dimxy->Width;
        *y=dimxy->Height;
        
return ;
} 

extern "C" __declspec(dllexport) void  irrgetimagesize( video::IImage * image, int * x, int * y)
{       
		const dimension2d<s32> * dimxy = &image->getDimension();
        *x=dimxy->Width;
        *y=dimxy->Height;
        
return ;
} 

extern "C" __declspec(dllexport) void irrrecalculateboundingbox ( SAnimatedMesh * sanimmesh )
{
    sanimmesh->recalculateBoundingBox();
return ;
}     


extern "C" __declspec(dllexport) u32 irrgetmeshjointcount( ISkinnedMesh * mesh )
{
return mesh->getJointCount();
} 


extern "C" __declspec(dllexport) char* irrgetmeshjointname( ISkinnedMesh * mesh , u32 i )
{
return (char*)mesh->getJointName(i);
} 

extern "C" __declspec(dllexport) ISceneNode* irrgetjointnode ( IAnimatedMeshSceneNode* node, u32 i )
{
    return node->getJointNode(i);
}

// see also irrcreatemesh in wrapperscene.cpp , irrrunning irrstop in wrappersystem.cpp

/* ----------------------------------------------------------------------------
grab a screenshot into a texture
*/
/*
extern "C" __declspec(dllexport) ITexture *  IrrGetScreenShot( u32 X, u32 Y, u32 W, u32 H )
{
    // grab the screenshot
	IImage *image = driver->createScreenShot();

	// create an image of the same size as the texture
	IImage *transitionImage = driver->createImage( image->getColorFormat(), dimension2d< u32 >( W, H ));

    // copy the pixels from the screenshot to the texture sized image
	image->copyTo( transitionImage, position2d<s32>(0,0), rect<s32>(X,Y,X+W,Y+H));

    // create a texture from the texture sized image
	return driver->addTexture("screenshot", transitionImage);
}



/*
IAnimatedMesh* plane = smgr->addHillPlaneMesh("floor", // Name of mesh 
core::dimension2d<f32>(1,1), //   Size of a tile of the mesh. 
core::dimension2d<u32>(100,100), //tileCount - Specifies how much tiles there will be. If you specifiy for example that a tile has the size (10.0f, 10.0f) and the tileCount is (10,10), than you get a field of 100 tiles which has the dimension 100.0fx100.0f. 
0, //material 
0, //hillheight 
core::dimension2d<f32>(10,10), //countHills 
core::dimension2d<f32>(100,100)); // texturerepeat 

ISceneNode* sea = smgr->addWaterSurfaceSceneNode(plane->getMesh(0), 2.0f, 350.0f, 8.0f,0,-1,vector3df(0,0,0),vector3df(10,0,0),vector3df(1,1,1)); 
sea->setMaterialTexture(0, driver->getTexture("c:/x/sand.png")); 
sea->setMaterialTexture(1, driver->getTexture("c:/x/water.png")); 
sea->setMaterialFlag(EMF_LIGHTING, true); 
sea->setMaterialType(video::EMT_REFLECTION_2_LAYER); 
*/ 


